@@ -874,8 +874,8 @@ class FactEditor(object):
         new = re.sub(re1, repl, s)
         old = re.sub(re2, '<font color="%s"><b>\\1</b></font>'
                      % clozeColour, s)
-        src.setHtml(new)
-        dst.setHtml(old)
+        src.setHtml(tidyHTML(new))
+        dst.setHtml(tidyHTML(old))
         self.lastCloze = (oldSrc, unicode(src.toHtml()),
                           unicode(dst.toHtml()))
         self.saveFields()
