@@ -50,6 +50,30 @@ class ChangeMap(QDialog):
             self.field = 0
         QDialog.accept(self)
 
+class UpdateMap(QDialog):
+    def __init__(self, parent, numFields, fieldModels):
+        QDialog.__init__(self, parent, Qt.Window)
+        self.parent = parent
+        self.fieldModels = fieldModels
+        self.dialog = ankiqt.forms.importup.Ui_Dialog()
+        self.dialog.setupUi(self)
+        self.connect(self.dialog.buttonBox.button(QDialogButtonBox.Help),
+                     SIGNAL("clicked()"), self.helpRequested)
+        for i in range(numFields):
+            self.dialog.fileField.addItem("Field %d" % (i+1))
+        for m in fieldModels:
+            self.dialog.deckField.addItem(m.name)
+        self.exec_()
+
+    def helpRequested(self):
+        QDesktopServices.openUrl(QUrl(ankiqt.appWiki + "FileImport"))
+
+    def accept(self):
+        self.updateKey = (
+            self.dialog.fileField.currentIndex(),
+            self.fieldModels[self.dialog.deckField.currentIndex()].id)
+        QDialog.accept(self)
+
 class ImportDialog(QDialog):
 
     def __init__(self, parent):
@@ -57,11 +81,8 @@ class ImportDialog(QDialog):
         self.parent = parent
         self.dialog = ankiqt.forms.importing.Ui_ImportDialog()
         self.dialog.setupUi(self)
-        self.tags = ui.tagedit.TagEdit(parent)
-        self.tags.setDeck(parent.deck)
-        self.dialog.topGrid.addWidget(self.tags,0,1,1,1)
-        self.setTabOrder(self.tags, self.dialog.tagDuplicates)
-        self.setTabOrder(self.dialog.tagDuplicates, self.dialog.autoDetect)
+        self.connect(self.dialog.buttonBox.button(QDialogButtonBox.Help),
+                     SIGNAL("clicked()"), self.helpRequested)
         self.setupMappingFrame()
         self.setupOptions()
         self.getFile()
@@ -83,6 +104,8 @@ class ImportDialog(QDialog):
         self.dialog.modelArea.setLayout(self.modelChooser)
         self.connect(self.dialog.importButton, SIGNAL("clicked()"),
                      self.doImport)
+        self.connect(self.dialog.updateButton, SIGNAL("clicked()"),
+                     self.doUpdate)
 
     def getFile(self):
         key = ";;".join([x[0] for x in importing.Importers])
@@ -103,10 +126,10 @@ class ImportDialog(QDialog):
         self.importerFunc = self.importer[1]
         if self.importerFunc.needMapper:
             self.modelChooser.show()
-            self.dialog.tagDuplicates.show()
         else:
             self.modelChooser.hide()
-            self.dialog.tagDuplicates.hide()
+            self.dialog.groupBox.setShown(False)
+            self.dialog.mappingGroup.setTitle("")
         self.dialog.autoDetect.setShown(self.importerFunc.needDelimiter)
 
     def maybePreview(self):
@@ -159,12 +182,20 @@ you can enter it here. Use \\t to represent tab."""),
             txt = _("Auto-detected &delimiter: %s") % d
         self.dialog.autoDetect.setText(txt)
 
-    def doImport(self):
+    def doUpdate(self):
+        f = UpdateMap(self,
+                      self.importer.fields(),
+                      self.model.fieldModels)
+        if not getattr(f, "updateKey", None):
+            # user cancelled
+            return
+        self.importer.updateKey = f.updateKey
+        self.doImport(True)
+
+    def doImport(self, update=False):
         self.dialog.status.setText(_("Importing..."))
         t = time.time()
         self.importer.mapping = self.mapping
-        self.importer.tagsToAdd = unicode(self.tags.text())
-        self.importer.tagDuplicates = self.dialog.tagDuplicates.isChecked()
         try:
             n = _("Import")
             self.parent.deck.setUndoStart(n)
@@ -194,16 +225,16 @@ you can enter it here. Use \\t to represent tab."""),
         self.file = None
         self.maybePreview()
         self.parent.deck.s.flush()
-        if sys.platform.startswith("win32") and not self.parent.deck.path:
-            # this fixes a strange bug in sqlite
-            self.parent.deck.s.all("pragma integrity_check")
         self.parent.reset()
         self.modelChooser.deinit()
 
     def setupMappingFrame(self):
         # qt seems to have a bug with adding/removing from a grid, so we add
         # to a separate object and add/remove that instead
-        self.mapbox = QVBoxLayout(self.dialog.mappingArea)
+        self.frame = QFrame(self.dialog.mappingArea)
+        self.dialog.mappingArea.setWidget(self.frame)
+        self.mapbox = QVBoxLayout(self.frame)
+        self.mapbox.setContentsMargins(0,0,0,0)
         self.mapwidget = None
 
     def hideMapping(self):
@@ -231,6 +262,7 @@ you can enter it here. Use \\t to represent tab."""),
             self.dialog.mappingArea.show()
         else:
             self.dialog.mappingArea.hide()
+            self.dialog.updateButton.hide()
             return
         # set up the mapping grid
         if self.mapwidget:
@@ -240,8 +272,8 @@ you can enter it here. Use \\t to represent tab."""),
         self.mapbox.addWidget(self.mapwidget)
         self.grid = QGridLayout(self.mapwidget)
         self.mapwidget.setLayout(self.grid)
-        self.grid.setMargin(6)
-        self.grid.setSpacing(12)
+        self.grid.setMargin(3)
+        self.grid.setSpacing(6)
         fields = self.importer.fields()
         for num in range(len(self.mapping)):
             text = _("Field <b>%d</b> of file is:") % (num + 1)
@@ -257,7 +289,6 @@ you can enter it here. Use \\t to represent tab."""),
             self.grid.addWidget(button, num, 2)
             self.connect(button, SIGNAL("clicked()"),
                          lambda s=self,n=num: s.changeMappingNum(n))
-        self.tags.setFocus()
 
     def changeMappingNum(self, n):
         f = ChangeMap(self.parent, self.model, self.mapping[n]).getField()
@@ -279,3 +310,6 @@ you can enter it here. Use \\t to represent tab."""),
     def reject(self):
         self.modelChooser.deinit()
         QDialog.reject(self)
+
+    def helpRequested(self):
+        QDesktopServices.openUrl(QUrl(ankiqt.appWiki + "FileImport"))
