@@ -33,16 +33,16 @@ class TextImporter(Importer):
         lineNum = 0
         ignored = 0
         if self.delimiter:
-            reader = csv.reader(self.data, delimiter=self.delimiter)
+            reader = csv.reader(self.data, delimiter=self.delimiter, doublequote=True)
         else:
-            reader = csv.reader(self.data, self.dialect)
+            reader = csv.reader(self.data, self.dialect, doublequote=True)
         for row in reader:
             try:
                 row = [unicode(x, "utf-8") for x in row]
             except UnicodeDecodeError, e:
                 raise ImportFormatError(
                     type="encodingError",
-                    info=_("The file was not in UTF8 format."))
+                    info=_("Please save in UTF-8 format. Click help for info."))
             if len(row) != self.numFields:
                 log.append(_(
                     "'%(row)s' had %(num1)d fields, "
@@ -74,9 +74,16 @@ class TextImporter(Importer):
         self.dialect = None
         self.fileobj = open(self.file, "rbU")
         self.data = self.fileobj.read()
-        self.data = re.sub("^ *#.*", "", self.data)
-        self.data = [x for x in self.data.split("\n") if x]
+        self.data = self.data.lstrip(codecs.BOM_UTF8)
+        def sub(s):
+            return re.sub(
+                "^\#.*", "", re.sub(
+                "^ +", "", s))
+        self.data = [sub(x) for x in self.data.split("\n") if sub(x)]
         if self.data:
+            if self.data[0].startswith("tags:"):
+                self.tagsToAdd = self.data[0][5:]
+                del self.data[0]
             self.updateDelimiter()
         if not self.dialect and not self.delimiter:
             raise ImportFormatError(
@@ -102,7 +109,7 @@ class TextImporter(Importer):
                     pass
         if self.dialect:
             try:
-                reader = csv.reader(self.data, self.dialect)
+                reader = csv.reader(self.data, self.dialect, doublequote=True)
             except:
                 err()
         else:
@@ -115,7 +122,7 @@ class TextImporter(Importer):
                     self.delimiter = ","
                 else:
                     self.delimiter = " "
-            reader = csv.reader(self.data, delimiter=self.delimiter)
+            reader = csv.reader(self.data, delimiter=self.delimiter, doublequote=True)
         try:
             self.numFields = len(reader.next())
         except:
