@@ -15,6 +15,7 @@ getDecks(): return a list of deck names & modtimes
 summary(lastSync): a list of all objects changed after lastSync
 applyPayload(payload): apply any sent changes and return any changed remote
                        objects
+finish(): save deck on server after payload applied and response received
 createDeck(name): create a deck on the server
 
 Full sync support is not documented yet.
@@ -22,9 +23,10 @@ Full sync support is not documented yet.
 __docformat__ = 'restructuredtext'
 
 import zlib, re, urllib, urllib2, socket, simplejson, time, shutil
-import os, base64, httplib, sys, tempfile, httplib
+import os, base64, httplib, sys, tempfile, httplib, types
 from datetime import date
 import anki, anki.deck, anki.cards
+from anki.db import sqlite
 from anki.errors import *
 from anki.models import Model, FieldModel, CardModel
 from anki.facts import Fact, Field
@@ -32,21 +34,24 @@ from anki.cards import Card
 from anki.stats import Stats, globalStats
 from anki.history import CardHistoryEntry
 from anki.stats import globalStats
-from anki.utils import ids2str, hexifyID
+from anki.utils import ids2str, hexifyID, checksum
+from anki.media import mediaFiles
 from anki.lang import _
 from hooks import runHook
 
 if simplejson.__version__ < "1.7.3":
-    raise "SimpleJSON must be 1.7.3 or later."
+    raise Exception("SimpleJSON must be 1.7.3 or later.")
 
+CHUNK_SIZE = 32768
 MIME_BOUNDARY = "Anki-sync-boundary"
 # live
-SYNC_URL = "http://anki.ichi2.net/sync/"
-SYNC_HOST = "anki.ichi2.net"; SYNC_PORT = 80
+SYNC_URL = "http://ankiweb.net/sync/"
+SYNC_HOST = "ankiweb.net"; SYNC_PORT = 80
 # testing
 #SYNC_URL = "http://localhost:8001/sync/"
 #SYNC_HOST = "localhost"; SYNC_PORT = 8001
 
+
 KEYS = ("models", "facts", "cards", "media")
 
 ##########################################################################
@@ -69,10 +74,12 @@ def incrementalSend(self, strOrFile):
             self.sock.sendall(strOrFile)
         else:
             cnt = 0
+            t = time.time()
             while 1:
-                if sendProgressHook:
+                if sendProgressHook and time.time() - t > 1:
                     sendProgressHook(cnt)
-                data = strOrFile.read(32768)
+                    t = time.time()
+                data = strOrFile.read(CHUNK_SIZE)
                 cnt += len(data)
                 if not data:
                     break
@@ -95,6 +102,7 @@ class SyncTools(object):
         self.deck = deck
         self.diffs = {}
         self.serverExcludedTags = []
+        self.timediff = 0
 
     # Control
     ##########################################################################
@@ -104,24 +112,24 @@ class SyncTools(object):
 
     def sync(self):
         "Sync two decks locally. Reimplement this for finer control."
-        if not self.prepareSync():
+        if not self.prepareSync(0):
             return
         sums = self.summaries()
         payload = self.genPayload(sums)
         res = self.server.applyPayload(payload)
         self.applyPayloadReply(res)
+        self.deck.reset()
 
-    def prepareSync(self):
+    def prepareSync(self, timediff):
         "Sync setup. True if sync needed."
         self.localTime = self.modified()
         self.remoteTime = self.server.modified()
         if self.localTime == self.remoteTime:
             return False
         l = self._lastSync(); r = self.server._lastSync()
-        if l != r:
-            self.deck.lastSync = min(l, r) - 600
-        else:
-            self.deck.lastSync = l
+        # set lastSync to the lower of the two sides, and account for slow
+        # clocks & assume it took up to 10 seconds for the reply to arrive
+        self.deck.lastSync = min(l, r) - timediff - 10
         return True
 
     def summaries(self):
@@ -141,11 +149,11 @@ class SyncTools(object):
             self.deleteObjsFromKey(diff[3], key)
         # handle the remainder
         if self.localTime > self.remoteTime:
-            payload['deck'] = self.bundleDeck()
             payload['stats'] = self.bundleStats()
             payload['history'] = self.bundleHistory()
             payload['sources'] = self.bundleSources()
-            self.deck.lastSync = self.deck.modified
+            # finally, set new lastSync and bundle the deck info
+            payload['deck'] = self.bundleDeck()
         return payload
 
     def applyPayload(self, payload):
@@ -162,11 +170,11 @@ class SyncTools(object):
                 self.deleteObjsFromKey(payload['deleted-' + key], key)
         # send back deck-related stuff if it wasn't sent to us
         if not 'deck' in payload:
-            reply['deck'] = self.bundleDeck()
             reply['stats'] = self.bundleStats()
             reply['history'] = self.bundleHistory()
             reply['sources'] = self.bundleSources()
-            self.deck.lastSync = self.deck.modified
+            # finally, set new lastSync and bundle the deck info
+            reply['deck'] = self.bundleDeck()
         else:
             self.updateDeck(payload['deck'])
             self.updateStats(payload['stats'])
@@ -174,12 +182,10 @@ class SyncTools(object):
             if 'sources' in payload:
                 self.updateSources(payload['sources'])
         self.postSyncRefresh()
-        # rebuild priorities on server
         cardIds = [x[0] for x in payload['added-cards']]
         self.deck.updateCardTags(cardIds)
+        # rebuild priorities on server
         self.rebuildPriorities(cardIds, self.serverExcludedTags)
-        # rebuild due counts
-        self.deck.rebuildCounts(full=False)
         return reply
 
     def applyPayloadReply(self, reply):
@@ -201,8 +207,14 @@ class SyncTools(object):
         cardIds = [x[0] for x in reply['added-cards']]
         self.deck.updateCardTags(cardIds)
         self.rebuildPriorities(cardIds)
-        # rebuild due counts
-        self.deck.rebuildCounts(full=False)
+        if self.missingFacts() != 0:
+            raise Exception(
+                "Facts missing after sync. Please run Tools>Advanced>Check DB.")
+
+    def missingFacts(self):
+        return self.deck.s.scalar(
+            "select count() from cards where factId "+
+                "not in (select id from facts)");
 
     def rebuildPriorities(self, cardIds, suspend=[]):
         self.deck.updateAllPriorities(partial=True, dirty=False)
@@ -254,36 +266,35 @@ class SyncTools(object):
         "Generate a full summary of modtimes for two-way syncing."
         # client may have selected an earlier sync time
         self.deck.lastSync = lastSync
-        self.deck.s.commit()
         # ensure we're flushed first
         self.deck.s.flush()
         return {
             # cards
-            "cards": self.realTuples(self.deck.s.all(
+            "cards": self.realLists(self.deck.s.all(
             "select id, modified from cards where modified > :mod",
             mod=lastSync)),
-            "delcards": self.realTuples(self.deck.s.all(
+            "delcards": self.realLists(self.deck.s.all(
             "select cardId, deletedTime from cardsDeleted "
             "where deletedTime > :mod", mod=lastSync)),
             # facts
-            "facts": self.realTuples(self.deck.s.all(
+            "facts": self.realLists(self.deck.s.all(
             "select id, modified from facts where modified > :mod",
             mod=lastSync)),
-            "delfacts": self.realTuples(self.deck.s.all(
+            "delfacts": self.realLists(self.deck.s.all(
             "select factId, deletedTime from factsDeleted "
             "where deletedTime > :mod", mod=lastSync)),
             # models
-            "models": self.realTuples(self.deck.s.all(
+            "models": self.realLists(self.deck.s.all(
             "select id, modified from models where modified > :mod",
             mod=lastSync)),
-            "delmodels": self.realTuples(self.deck.s.all(
+            "delmodels": self.realLists(self.deck.s.all(
             "select modelId, deletedTime from modelsDeleted "
             "where deletedTime > :mod", mod=lastSync)),
             # media
-            "media": self.realTuples(self.deck.s.all(
+            "media": self.realLists(self.deck.s.all(
             "select id, created from media where created > :mod",
             mod=lastSync)),
-            "delmedia":  self.realTuples(self.deck.s.all(
+            "delmedia":  self.realLists(self.deck.s.all(
             "select mediaId, deletedTime from mediaDeleted "
             "where deletedTime > :mod", mod=lastSync)),
             }
@@ -394,6 +405,7 @@ class SyncTools(object):
 
     def getModel(self, id, create=True):
         "Return a local model with same ID, or create."
+        id = int(id)
         for l in self.deck.models:
             if l.id == id:
                 return l
@@ -414,8 +426,9 @@ class SyncTools(object):
                 self.deck.deleteFieldModel(model, fm)
 
     def getFieldModel(self, model, remote):
+        id = int(remote['id'])
         for fm in model.fieldModels:
-            if fm.id == remote['id']:
+            if fm.id == id:
                 return fm
         fm = FieldModel()
         model.addFieldModel(fm)
@@ -434,8 +447,9 @@ class SyncTools(object):
                 self.deck.deleteCardModel(model, cm)
 
     def getCardModel(self, model, remote):
+        id = int(remote['id'])
         for cm in model.cardModels:
-            if cm.id == remote['id']:
+            if cm.id == id:
                 return cm
         cm = CardModel()
         model.addCardModel(cm)
@@ -457,10 +471,10 @@ class SyncTools(object):
             modified = "modified"
         factIds = ids2str(ids)
         return {
-            'facts': self.realTuples(self.deck.s.all("""
+            'facts': self.realLists(self.deck.s.all("""
 select id, modelId, created, %s, tags, spaceUntil, lastCardId from facts
 where id in %s""" % (modified, factIds))),
-            'fields': self.realTuples(self.deck.s.all("""
+            'fields': self.realLists(self.deck.s.all("""
 select id, factId, fieldModelId, ordinal, value from fields
 where factId in %s""" % factIds))
             }
@@ -477,12 +491,9 @@ where factId in %s""" % factIds))
             'created': f[2],
             'modified': f[3],
             'tags': f[4],
-            'spaceUntil': f[5],
+            'spaceUntil': f[5] or "",
             'lastCardId': f[6]
             } for f in facts]
-        self.deck.factCount += (len(facts) - self.deck.s.scalar(
-            "select count(*) from facts where id in %s" %
-            ids2str([f[0] for f in facts])))
         self.deck.s.execute("""
 insert or replace into facts
 (id, modelId, created, modified, tags, spaceUntil, lastCardId)
@@ -517,18 +528,27 @@ values
     ##########################################################################
 
     def getCards(self, ids):
-        return self.realTuples(self.deck.s.all("""
+        return self.realLists(self.deck.s.all("""
 select id, factId, cardModelId, created, modified, tags, ordinal,
 priority, interval, lastInterval, due, lastDue, factor,
 firstAnswered, reps, successive, averageTime, reviewTime, youngEase0,
 youngEase1, youngEase2, youngEase3, youngEase4, matureEase0,
 matureEase1, matureEase2, matureEase3, matureEase4, yesCount, noCount,
-question, answer, lastFactor, spaceUntil, type, combinedDue
+question, answer, lastFactor, spaceUntil, type, combinedDue, relativeDelay
 from cards where id in %s""" % ids2str(ids)))
 
     def updateCards(self, cards):
         if not cards:
             return
+        # FIXME: older clients won't send this, so this is temp compat code
+        def getType(row):
+            if len(row) > 36:
+                return row[36]
+            if row[15]:
+                return 1
+            elif row[14]:
+                return 0
+            return 2
         dlist = [{'id': c[0],
                   'factId': c[1],
                   'cardModelId': c[2],
@@ -565,10 +585,8 @@ from cards where id in %s""" % ids2str(ids)))
                   'spaceUntil': c[33],
                   'type': c[34],
                   'combinedDue': c[35],
+                  'rd': getType(c)
                   } for c in cards]
-        self.deck.cardCount += (len(cards) - self.deck.s.scalar(
-            "select count(*) from cards where id in %s" %
-            ids2str([c[0] for c in cards])))
         self.deck.s.execute("""
 insert or replace into cards
 (id, factId, cardModelId, created, modified, tags, ordinal,
@@ -585,7 +603,7 @@ values
 :youngEase1, :youngEase2, :youngEase3, :youngEase4, :matureEase0,
 :matureEase1, :matureEase2, :matureEase3, :matureEase4, :yesCount,
 :noCount, :question, :answer, :lastFactor, :spaceUntil,
-:type, :combinedDue, 0, 0)""", dlist)
+:type, :combinedDue, :rd, 0)""", dlist)
         self.deck.s.statement(
             "delete from cardsDeleted where cardId in %s" %
             ids2str([c[0] for c in cards]))
@@ -597,6 +615,12 @@ values
     ##########################################################################
 
     def bundleDeck(self):
+        # ensure modified is not greater than server time
+        if getattr(self, "server", None) and getattr(
+            self.server, "timestamp", None):
+            self.deck.modified = min(self.deck.modified,self.server.timestamp)
+        # and ensure lastSync is greater than modified
+        self.deck.lastSync = max(time.time(), self.deck.modified+1)
         d = self.dictFromObj(self.deck)
         del d['Session']
         del d['engine']
@@ -604,10 +628,19 @@ values
         del d['path']
         del d['syncName']
         del d['version']
+        if 'newQueue' in d:
+            del d['newQueue']
+            del d['failedQueue']
+            del d['revQueue']
         # these may be deleted before bundling
+        if 'css' in d: del d['css']
         if 'models' in d: del d['models']
         if 'currentModel' in d: del d['currentModel']
-        d['meta'] = self.realTuples(self.deck.s.all("select * from deckVars"))
+        keys = d.keys()
+        for k in keys:
+            if isinstance(d[k], types.MethodType):
+                del d[k]
+        d['meta'] = self.realLists(self.deck.s.all("select * from deckVars"))
         return d
 
     def updateDeck(self, deck):
@@ -619,8 +652,6 @@ insert or replace into deckVars
 (key, value) values (:k, :v)""", k=k, v=v)
             del deck['meta']
         self.applyDict(self.deck, deck)
-        self.deck.lastSync = self.deck.modified
-        self.deck.updateDynamicIndices()
 
     def bundleStats(self):
         def bundleStat(stat):
@@ -659,7 +690,7 @@ insert or replace into deckVars
             stat.toDB(self.deck.s)
 
     def bundleHistory(self):
-        return self.realTuples(self.deck.s.all("""
+        return self.realLists(self.deck.s.all("""
 select cardId, time, lastInterval, nextInterval, ease, delay,
 lastFactor, nextFactor, reps, thinkingTime, yesCount, noCount
 from reviewHistory where time > :ls""",
@@ -690,7 +721,7 @@ values
                          dlist)
 
     def bundleSources(self):
-        return self.realTuples(self.deck.s.all("select * from sources"))
+        return self.realLists(self.deck.s.all("select * from sources"))
 
     def updateSources(self, sources):
         for s in sources:
@@ -751,6 +782,7 @@ where media.id in %s""" % sids, now=time.time())
         "Sync two decks one way."
         payload = self.server.genOneWayPayload(lastSync)
         self.applyOneWayPayload(payload)
+        self.deck.reset()
 
     def syncOneWayDeckName(self):
         return (self.deck.s.scalar("select name from sources where id = :id",
@@ -785,7 +817,7 @@ where media.id in %s""" % sids, now=time.time())
         # cards
         cardIds = self.deck.s.column0(
             "select id from cards where modified > :l", l=lastSync)
-        p['cards'] = self.realTuples(self.getOneWayCards(cardIds))
+        p['cards'] = self.realLists(self.getOneWayCards(cardIds))
         return p
 
     def applyOneWayPayload(self, payload):
@@ -804,7 +836,8 @@ where media.id in %s""" % sids, now=time.time())
         try:
             self.updateOneWayCards(payload['cards'])
         except KeyError:
-            sys.stderr.write("Error in one way sync\n")
+            sys.stderr.write("Subscribed to a broken deck. "
+                             "Try removing your deck subscriptions.")
             t = 0
         # update sync time
         self.deck.s.statement(
@@ -824,9 +857,6 @@ where media.id in %s""" % sids, now=time.time())
         t = time.time()
         dlist = [{'id': c[0], 'factId': c[1], 'cardModelId': c[2],
                   'ordinal': c[3], 'created': c[4], 't': t} for c in cards]
-        self.deck.cardCount += (len(cards) - self.deck.s.scalar(
-            "select count(*) from cards where id in %s" %
-            ids2str([c[0] for c in cards])))
         # add any missing cards
         self.deck.s.statements("""
 insert or ignore into cards
@@ -843,7 +873,7 @@ values
 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0,
-0, "", "", 2.5, 0, 0, 2, :t, 0)""", dlist)
+0, "", "", 2.5, 0, 0, 2, :t, 2)""", dlist)
         # update q/as
         models = dict(self.deck.s.all("""
 select cards.id, models.id
@@ -886,9 +916,9 @@ and cards.id in %s""" % ids2str([c[0] for c in cards])))
         for (k,v) in dict.items():
             setattr(obj, k, v)
 
-    def realTuples(self, result):
-        "Convert an SQLAlchemy response into a list of real tuples."
-        return [tuple(x) for x in result]
+    def realLists(self, result):
+        "Convert an SQLAlchemy response into a list of real lists."
+        return [list(x) for x in result]
 
     def getObjsFromKey(self, ids, key):
         return getattr(self, "get" + key.capitalize())(ids)
@@ -907,11 +937,11 @@ and cards.id in %s""" % ids2str([c[0] for c in cards])))
             return True
         for sum in sums:
             for l in sum.values():
-                if len(l) > 500:
+                if len(l) > 1000:
                     return True
         if self.deck.s.scalar(
             "select count() from reviewHistory where time > :ls",
-            ls=self.deck.lastSync) > 500:
+            ls=self.deck.lastSync) > 1000:
             return True
         lastDay = date.fromtimestamp(max(0, self.deck.lastSync - 60*60*24))
         if self.deck.s.scalar(
@@ -922,7 +952,8 @@ and cards.id in %s""" % ids2str([c[0] for c in cards])))
 
     def prepareFullSync(self):
         t = time.time()
-        self.deck.lastSync = t
+        # ensure modified is not greater than server time
+        self.deck.modified = min(self.deck.modified, self.server.timestamp)
         self.deck.s.commit()
         self.deck.close()
         fields = {
@@ -965,11 +996,12 @@ and cards.id in %s""" % ids2str([c[0] for c in cards])))
             # data
             comp = zlib.compressobj()
             while 1:
-                data = src.read(32768)
+                data = src.read(CHUNK_SIZE)
                 if not data:
                     tmp.write(comp.flush())
                     break
                 tmp.write(comp.compress(data))
+            src.close()
             tmp.write('\r\n--' + MIME_BOUNDARY + '--\r\n\r\n')
             size = tmp.tell()
             tmp.seek(0)
@@ -981,10 +1013,17 @@ and cards.id in %s""" % ids2str([c[0] for c in cards])))
                 'Content-length': str(size),
                 'Host': SYNC_HOST,
                 }
-            req = urllib2.Request(SYNC_URL + "fullup", tmp, headers)
+            req = urllib2.Request(SYNC_URL + "fullup?v=2", tmp, headers)
             try:
                 sendProgressHook = fullSyncProgressHook
-                assert urllib2.urlopen(req).read() == "OK"
+                res = urllib2.urlopen(req).read()
+                assert res.startswith("OK")
+                # update lastSync
+                c = sqlite.connect(path)
+                c.execute("update decks set lastSync = ?",
+                          (res[3:],))
+                c.commit()
+                c.close()
             finally:
                 sendProgressHook = None
                 tmp.close()
@@ -1004,12 +1043,12 @@ and cards.id in %s""" % ids2str([c[0] for c in cards])))
             decomp = zlib.decompressobj()
             cnt = 0
             while 1:
-                data = src.read(32768)
+                data = src.read(CHUNK_SIZE)
                 if not data:
                     tmp.write(decomp.flush())
                     break
                 tmp.write(decomp.decompress(data))
-                cnt += 32768
+                cnt += CHUNK_SIZE
                 runHook("fullSyncProgress", "fromServer", cnt)
             src.close()
             tmp.close()
@@ -1017,6 +1056,12 @@ and cards.id in %s""" % ids2str([c[0] for c in cards])))
             # if we were successful, overwrite old deck
             os.unlink(path)
             os.rename(tmpname, path)
+            # reset the deck name
+            c = sqlite.connect(path)
+            c.execute("update decks set syncName = ?",
+                      [checksum(path.encode("utf-8"))])
+            c.commit()
+            c.close()
         finally:
             runHook("fullSyncFinished")
 
@@ -1062,6 +1107,7 @@ class HttpSyncServerProxy(SyncServer):
                 raise SyncError(type="authFailed", status=d['status'])
             self.decks = d['decks']
             self.timestamp = d['timestamp']
+            self.timediff = abs(self.timestamp - time.time())
 
     def hasDeck(self, deckName):
         self.connect()
@@ -1097,9 +1143,13 @@ class HttpSyncServerProxy(SyncServer):
         return self.runCmd("applyPayload",
                            payload=self.stuff(payload))
 
+    def finish(self):
+        assert self.runCmd("finish") == "OK"
+
     def runCmd(self, action, **args):
         data = {"p": self.password,
-                "u": self.username}
+                "u": self.username,
+                "v": 2}
         if self.deckName:
             data['d'] = self.deckName.encode("utf-8")
         else:
@@ -1109,12 +1159,17 @@ class HttpSyncServerProxy(SyncServer):
         try:
             f = urllib2.urlopen(SYNC_URL + action, data)
         except (urllib2.URLError, socket.error, socket.timeout,
-                httplib.BadStatusLine):
-            raise SyncError(type="noResponse")
+                httplib.BadStatusLine), e:
+            raise SyncError(type="connectionError",
+                            exc=`e`)
         ret = f.read()
         if not ret:
             raise SyncError(type="noResponse")
-        return self.unstuff(ret)
+        try:
+            return self.unstuff(ret)
+        except Exception, e:
+            raise SyncError(type="connectionError",
+                            exc=`e`)
 
 # HTTP server: respond to proxy requests and return data
 ##########################################################################
@@ -1152,14 +1207,27 @@ class HttpSyncServer(SyncServer):
 ##########################################################################
 
 def copyLocalMedia(src, dst):
-    src = src.mediaDir()
-    if not src:
+    srcDir = src.mediaDir()
+    if not srcDir:
         return
-    dst = dst.mediaDir(create=True)
-    files = os.listdir(src)
+    dstDir = dst.mediaDir(create=True)
+    files = os.listdir(srcDir)
+    # find media references
+    used = {}
+    for col in ("question", "answer"):
+        txt = dst.s.column0("""
+select %(c)s from cards where
+%(c)s like '%%<img %%'
+or %(c)s like '%%[sound:%%'""" % {'c': col})
+        for entry in txt:
+            for fname in mediaFiles(entry):
+                used[fname] = True
+    # copy only used media
     for file in files:
-        srcfile = os.path.join(src, file)
-        dstfile = os.path.join(dst, file)
+        if file not in used:
+            continue
+        srcfile = os.path.join(srcDir, file)
+        dstfile = os.path.join(dstDir, file)
         if not os.path.exists(dstfile):
             try:
                 shutil.copy2(srcfile, dstfile)
